`timescale 1ns/1ps

module Camera_UDP_GETH_tb;

	reg cmos_pclk;
	reg GMII_GTXC;
	reg Rst_n;
	reg Init_Done;
	reg cmos_vsync;
	reg	cmos_href;
	reg	[7:0] cmos_data;
	
	wire [7:0] GMII_TXD;
	wire GMII_TXEN;
	
	wire fifo_aclr;
	wire fifo_wrreq;
	wire [7:0] fifo_wrdata;
	wire [12:0] fifo_usedw;
	
	parameter IMG_WIDTH = 20;
	
	cmos_capture_rgb565 cmos_capture_rgb565(
		.cmos_pclk(cmos_pclk),
		.Rst_n(Rst_n),
		.Init_Done(Init_Done),
		.cmos_vsync(cmos_vsync),
		.cmos_href(cmos_href),
		.cmos_data(cmos_data),
		.fifo_aclr(fifo_aclr),
		.fifo_wrreq(fifo_wrreq),
		.fifo_wrdata(fifo_wrdata)
	);
	UDP_Send UDP_Send(
		.Clk(),
		.GMII_GTXC(GMII_GTXC),
		.GMII_TXD(GMII_TXD),
		.GMII_TXEN(GMII_TXEN),
		.Rst_n(Init_Done),
		.Tx_Done(),
		.data_length(IMG_WIDTH*2+2),
		.des_ip(32'hc0_a8_00_03),
		.des_mac(48'hFF_FF_FF_FF_FF_FF),
		.des_port(16'd6000),
		.src_ip(32'hc0_a8_00_02),
		.src_mac(48'h00_0a_35_01_fe_c0),
		.src_port(16'd5000),
		.wrclk(cmos_pclk),
		.wrdata(fifo_wrdata),
		.wrreq(fifo_wrreq),
		.aclr(fifo_aclr),
		.wrusedw(fifo_usedw)
	);
	
	integer i;
	
	always #4 GMII_GTXC = ~GMII_GTXC;		//125M
	always #6 cmos_pclk = ~cmos_pclk;	//84M
	
	initial begin
		GMII_GTXC = 1;
		cmos_pclk = 1;
		Rst_n = 0;
		
		Init_Done = 0;
		
		cmos_vsync = 0;
		cmos_href = 0;
		cmos_data = 8'd0;
		
		#202
		Rst_n = 1;
		
		Init_Done = 1;
		
		repeat (6) begin
			photo;
		end
		
		#2000;
		$stop;
	end
	
	task frame;
		begin
			cmos_href = 1;
			for (i = 0; i < IMG_WIDTH; i = i + 1) begin
				cmos_data = cmos_data + 1;
				#12;
			end
			cmos_href = 0;
			#120;
		end
	endtask
	
	task photo;
		begin
			cmos_vsync = 0;
			
			repeat (15) begin
				cmos_data = 8'd0;
				frame;
			end
			cmos_vsync = 1;
			#12;
		end
	endtask

endmodule
