`timescale 1ns/1ps
//-----------------------------------------------------
//Sensor HS & VS Vaild Capture
/************************************************************
mode : compression mode 3
										        ____
帧:	VS________________________...______________|    |________
		  _______   _______	 	     ___
行:	HS___|       |_|       |__...___|   |____________________
************************************************************/
//-------------------------------------------------------------
module cmos_capture_rgb565(
	cmos_pclk,
	Rst_n,
	Init_Done,
	cmos_vsync,
	cmos_href,
	cmos_data,
	
	fifo_aclr,
	fifo_wrreq,
	fifo_wrdata
);
	input cmos_pclk;
	input Rst_n;
	input Init_Done;
	input cmos_vsync;
	input cmos_href;
	input [7:0] cmos_data;
	output reg fifo_aclr;
	output fifo_wrreq;
	output [7:0] fifo_wrdata;

	reg       ir_cmos_vsync;
	reg       ir_cmos_href;
	reg [7:0] ir_cmos_data;

	//同步时钟域
	wire cmos_pclk;
	always @ (posedge cmos_pclk) begin
		ir_cmos_vsync <= #1 cmos_vsync;
		ir_cmos_href <= #1 cmos_href;
		ir_cmos_data <= #1 cmos_data;
	end
	
	always @ (posedge cmos_pclk or negedge Rst_n)
	if (!Rst_n)
		fifo_aclr <= #1 1'b1;
	else if (Init_Done && ir_cmos_vsync) //等到初始化摄像完成且头场同步信号出现，释放清零信号，开始写入数据
		fifo_aclr <= #1 1'b0;
	else
		fifo_aclr <= #1 fifo_aclr;
	
	reg [1:0] cmos_href_r;
	reg nedge_cmos_href;
	always @ (posedge cmos_pclk)
		cmos_href_r <= #1 {cmos_href_r[0], ir_cmos_href};

	always @ (posedge cmos_pclk)
		nedge_cmos_href <= (cmos_href_r == 2'b10);
	
	reg [15:0] vcnt; //行计数器
	always @ (posedge cmos_pclk or negedge Rst_n)
	if (!Rst_n)
		vcnt <= #1 0;
	else if (ir_cmos_vsync)	//检测到场同步信号，行计数器清零
		vcnt <= #1 0;
	else if (nedge_cmos_href)
		vcnt <= #1 vcnt + 1'b1;
	
	reg [23:0] data_tmp;
	always @ (posedge cmos_pclk)
	if (cmos_href_r[1] | cmos_href_r[0] | ir_cmos_href)
		data_tmp <= #1 {data_tmp[15:0], ir_cmos_data};
	else
		data_tmp <= #1 {ir_cmos_data, vcnt[7:0], vcnt[15:8]};
	
	assign fifo_wrreq = (fifo_aclr == 1'b0) ? cmos_href_r[1] | cmos_href_r[0] | nedge_cmos_href : 1'b0;
	assign fifo_wrdata = (fifo_aclr == 1'b0) ? data_tmp[23:16] : 16'd0;
	
endmodule
